==> emelfm2-0.1.2.tar.gz: emelfm2-0.1.2/plugins/e2p_pack.c <==
/*
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
#include "e2_plugins.h"
#include "e2_dialog.h"
#include "e2_command.h"
#include "e2_filelist.h"

static GtkWidget *filename_entry;
static GtkWidget *pkgtype_combo;

enum { TAR_GZ, TAR_BZ2, TAR, ZIP, RAR, ARJ, MAXTYPES };

static gboolean _e2p_pack_no_cb (GtkWidget *widget, GtkWidget *dialog)
{
  gtk_widget_set_sensitive (app.main_window, TRUE);
  gtk_widget_destroy (dialog);
  gtk_main_quit ();
  return TRUE;
}

static gboolean _e2p_pack_delete_cb (GtkWidget *widget,
	GdkEvent *event, GtkWidget *dialog)
{
    return _e2p_pack_no_cb (widget, dialog);
}

static gboolean _e2p_pack_key_press_cb (GtkWidget *widget,
	GdkEventKey *event, GtkWidget *dialog)
{
  if (event->keyval == GDK_Escape)
    return _e2p_pack_no_cb (widget, dialog);
  return FALSE;
}

static gboolean _e2p_pack_yes_cb (GtkWidget *widget, GtkWidget *dialog)
{
  static gchar *name_str [MAXTYPES] =
  {	//these extension strings are in same order as enum
    ".tar.gz",
    ".tar.bz2",
    ".tar",
    ".zip",
	".rar",
	".arj"
  };
  static gchar *cmd_str [MAXTYPES] =
  {	//these command strings are in same order as enum
    ">tar cvf - %%f | gzip - > %s",	//run in separate shell
    ">tar cvf - %%f | bzip2 - > %s",	//ditto
    "tar cvf %s %%f",
    "zip -r %s %%f",
	"rar a -r -ol %s %%f",
	"arj a -a1 -r %s %%f"
  };

  gint pkg_type = gtk_combo_box_get_active (GTK_COMBO_BOX (pkgtype_combo));
  if (pkg_type != -1)
  {
    const gchar *chosen_name = gtk_entry_get_text (GTK_ENTRY (filename_entry));
    gchar *full_name = g_strconcat (chosen_name, name_str [pkg_type], NULL);
  	gboolean flag = TRUE;
	if (e2_option_bool_get ("confirm-overwrite"))
	{
		gchar *utf = g_strdup_printf ("%s%s", curr_view->dir, full_name);  //separator comes with dir
		F_DECLARE;
    	gchar *local = F_FILENAME_TO_LOCALE (utf);
		if (e2_fs_access2 (local) == 0)
		{	//same-named item exists already
			DialogButtons result = e2_dialog_ow_check (full_name, NONE);
			if (result != OK)
				flag = FALSE;	//signal that we will not proceed
		}
		g_free (utf);
		F_FREE (local);
	}
	if (flag)
	{
		gchar *command =  g_strdup_printf (cmd_str [pkg_type], full_name);
		e2_filelist_disable_refresh ();	//prevent changes to selected item data
		e2_command_run (command, E2_COMMAND_RANGE_DEFAULT);
		e2_filelist_enable_refresh ();
    	g_free (command);
	}
	g_free (full_name);
  }

  return _e2p_pack_no_cb (widget, dialog);
}

static void _e2p_pack (void)
{
  GtkWidget *dialog;
  GtkWidget *dialog_vbox;
  GtkWidget *hbox;

  FileInfo *info = e2_fileview_get_selected_first_local (curr_view);
  if (info == NULL)
	return;
  gchar *suggested_name = D_FILENAME_FROM_LOCALE (info->filename);
  gchar *s = strrchr (suggested_name, '.');	//always ascii '.', don't need g_utf8_strrchr()
  if (s != NULL)
	  *s = '\0';

  dialog = gtk_dialog_new ();
  E2_DIALOG_TITLE (dialog, _("archive creation"));
  dialog_vbox = GTK_DIALOG(dialog)->vbox;
  gtk_container_set_border_width (GTK_CONTAINER(dialog_vbox), E2_PADDING_LARGE);

  hbox = e2_widget_add_box (dialog_vbox, TRUE, 0, FALSE, FALSE, 0);
  _e2_widget_add_label (hbox, _("Filename: "), 0, 0, FALSE, 0);

  filename_entry = e2_widget_add_entry (hbox, suggested_name, TRUE, TRUE);
  g_free (suggested_name);
  pkgtype_combo = gtk_combo_box_entry_new_text ();
  //these package name strings are in same order as enum
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".tar.gz"));  //index 0
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".tar.bz2"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".tar"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".zip"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".rar"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".arj"));  //index 5
  gtk_combo_box_set_active (GTK_COMBO_BOX (pkgtype_combo), 0);
  gtk_widget_set_size_request (pkgtype_combo, 120, -1);  //'natural' width (= filename entry width) too big
  gtk_box_pack_start (GTK_BOX (hbox), pkgtype_combo, TRUE, TRUE, 0);
  gtk_widget_show (pkgtype_combo);

  g_signal_connect (G_OBJECT (dialog), "delete_event",
	G_CALLBACK (_e2p_pack_delete_cb), dialog);
  g_signal_connect (G_OBJECT (dialog), "key_press_event",
	G_CALLBACK (_e2p_pack_key_press_cb), dialog);

  e2_dialog_add_button_custom (dialog, FALSE, &E2_BUTTON_CANCEL,
	NULL, _e2p_pack_no_cb, dialog);
  e2_dialog_add_button_custom (dialog, TRUE, &E2_BUTTON_OK,
	NULL, _e2p_pack_yes_cb, dialog);

  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
  gtk_widget_show (dialog);
  gtk_widget_set_sensitive (app.main_window, FALSE);

  gtk_main();
}

//aname must be confined to this module
static gchar *aname;
/**
 * @brief plugin initialization function, called by main program
 *
 * @param p ptr to plugin data struct
 *
 * @return TRUE if the initialization succeeds, else FALSE
 **/
gboolean init_plugin (Plugin *p)
{
#define ANAME "pack"
  aname = _("pack");

  p->signature = ANAME VERSION;
  p->menu_name = _("_Pack..");
  p->description = _("Build an archive containing the selected item(s)");
  p->icon = ICON_DIR"plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate

  if (p->plugin_cb == NULL)
  {
	  p->plugin_cb = _e2p_pack;  //set plugins menu item function
//	  E2_Action* (*action_register) () = (*p->api_lookup) (E2API_ACTION_REGISTER);
	  //no need to free this
	  gchar *action_name = g_strconcat (PLUGIN,".",aname,NULL);
	  //NULL = no data, FALSE = no argument for the function
//	  (*action_register)
	  e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2p_pack, NULL, FALSE);
	  return TRUE;
  }
  return FALSE;
}
/**
 * @brief unload this plugin
 *
 * @param p pointer to data struct for the plugin
 *
 * @return
 **/
void unload_plugin (Plugin *p)
{
//  if (p->api_lookup != NULL)
//  {
//	  gboolean (*action_unregister) () = (*p->api_lookup) (E2API_ACTION_UNREGISTER);
	  gchar *action_name = g_strconcat (PLUGIN,".",aname,NULL);
//	  (*action_unregister)
	  e2_action_unregister (action_name);
	  g_free (action_name);
//  }
}

==> emelfm2-0.1.4.tar.gz: emelfm2-0.1.4/plugins/e2p_pack.c <==
/* $Id: e2p_pack.c 1450 2005-12-21 11:22:10Z tpgww $
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
#include "e2_plugins.h"
#include "e2_dialog.h"
#include "e2_command.h"
#include "e2_filelist.h"

static GtkWidget *filename_entry;
static GtkWidget *pkgtype_combo;

enum { TAR_GZ, TAR_BZ2, TAR, ZIP, RAR, ARJ, MAXTYPES };

static gboolean _e2p_pack_no_cb (GtkWidget *widget, GtkWidget *dialog)
{
  gtk_widget_set_sensitive (app.main_window, TRUE);
  gtk_widget_destroy (dialog);
  gtk_main_quit ();
  return TRUE;
}

static gboolean _e2p_pack_yes_cb (GtkWidget *widget, GtkWidget *dialog)
{
  static gchar *name_str [MAXTYPES] =
  {	//these extension strings are in same order as enum
    ".tar.gz",
    ".tar.bz2",
    ".tar",
    ".zip",
	".rar",
	".arj"
  };
  static gchar *cmd_str [MAXTYPES] =
  {	//these command strings are in same order as enum
	  //NOTE that %%f is converted to %f when the command is constructed with g_strdup_printf ()
    ">tar cvf - %%f | gzip - > \"%s\"",	//run in separate shell
    ">tar cvf - %%f | bzip2 - > \"%s\"",	//ditto
    "tar cvf \"%s\" %%f",
    "zip -r \"%s\" %%f",
	"rar a -r -ol \"%s\" %%f",
	"arj a -a1 -r \"%s\" %%f"
  };

  gint pkg_type = gtk_combo_box_get_active (GTK_COMBO_BOX (pkgtype_combo));
  if (pkg_type != -1)
  {
	const gchar *chosen_name = gtk_entry_get_text (GTK_ENTRY (filename_entry));
	gboolean flag = (*chosen_name != '\0');
	gchar *full_name;
	if (flag)
	{
		full_name = g_strconcat (chosen_name, name_str [pkg_type], NULL);
		if (e2_option_bool_get ("confirm-overwrite"))
		{
			gchar *utf = g_strdup_printf ("%s%s", curr_view->dir, full_name);  //separator comes with dir
    		gchar *local = F_FILENAME_TO_LOCALE (utf);
			if (e2_fs_access2 (local) == 0)
			{	//same-named item exists already
				DialogButtons result = e2_dialog_ow_check (full_name, NONE);
				if (result != OK)
				{
					flag = FALSE;	//signal that we will not proceed
					g_free (full_name);
				}
			}
			g_free (utf);
			F_FREE (local);
		}
	}
	if (flag)
	{
		gchar *command =  g_strdup_printf (cmd_str [pkg_type], full_name);
		e2_filelist_disable_refresh ();	//prevent changes to selected item data
		e2_command_run (command, E2_COMMAND_RANGE_DEFAULT);
		e2_filelist_enable_refresh ();
    	g_free (command);
		g_free (full_name);
	}
  }

  return _e2p_pack_no_cb (widget, dialog);
}

static gboolean _e2p_pack_delete_cb (GtkWidget *widget,
	GdkEvent *event, GtkWidget *dialog)
{
    return _e2p_pack_no_cb (widget, dialog);
}

static gboolean _e2p_pack_key_press_cb (GtkWidget *widget,
	GdkEventKey *event, GtkWidget *dialog)
{
  if (event->keyval == GDK_Return)
	  return _e2p_pack_yes_cb (widget, dialog);
  else if (event->keyval == GDK_Escape)
    return _e2p_pack_no_cb (widget, dialog);
	  
  return FALSE;
}

static void _e2p_pack (void)
{
  GtkWidget *dialog;
  GtkWidget *dialog_vbox;
  GtkWidget *hbox;

  FileInfo *info = e2_fileview_get_selected_first_local (curr_view);
  if (info == NULL)
	return;

  dialog = gtk_dialog_new ();
  E2_DIALOG_TITLE (dialog, _("archive creation"));
  dialog_vbox = GTK_DIALOG(dialog)->vbox;
  gtk_container_set_border_width (GTK_CONTAINER(dialog_vbox), E2_PADDING_LARGE);

  hbox = e2_widget_add_box (dialog_vbox, TRUE, 0, FALSE, FALSE, 0);
  _e2_widget_add_label (hbox, _("Filename: "), 0, 0, FALSE, 0);

//  gchar *suggested_name = D_FILENAME_FROM_LOCALE (info->filename);
//  gchar *s = strrchr (suggested_name, '.');	//always ascii '.', don't need g_utf8_strrchr()
//  if (s != NULL)
//	  *s = '\0';
  gchar *suggested_name = F_FILENAME_FROM_LOCALE (info->filename);
  filename_entry = e2_widget_add_entry (hbox, suggested_name, TRUE, TRUE);
//  g_free (suggested_name);
  F_FREE (suggested_name);
  pkgtype_combo = gtk_combo_box_entry_new_text ();
  //these package name strings are in same order as enum
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".tar.gz"));  //index 0
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".tar.bz2"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".tar"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".zip"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".rar"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".arj"));  //index 5
  gtk_combo_box_set_active (GTK_COMBO_BOX (pkgtype_combo), 0);
  gtk_widget_set_size_request (pkgtype_combo, 120, -1);  //'natural' width (= filename entry width) too big
  gtk_box_pack_start (GTK_BOX (hbox), pkgtype_combo, TRUE, TRUE, 0);
  gtk_widget_show (pkgtype_combo);

  g_signal_connect (G_OBJECT (dialog), "delete_event",
	G_CALLBACK (_e2p_pack_delete_cb), dialog);
  g_signal_connect (G_OBJECT (dialog), "key_press_event",
	G_CALLBACK (_e2p_pack_key_press_cb), dialog);

  e2_dialog_add_button_custom (dialog, FALSE, &E2_BUTTON_CANCEL,
	NULL, _e2p_pack_no_cb, dialog);
  e2_dialog_add_button_custom (dialog, TRUE, &E2_BUTTON_OK,
	NULL, _e2p_pack_yes_cb, dialog);

  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
  gtk_widget_show (dialog);
  gtk_widget_set_sensitive (app.main_window, FALSE);

  gtk_main();
}

//aname must be confined to this module
static gchar *aname;
/**
 * @brief plugin initialization function, called by main program
 *
 * @param p ptr to plugin data struct
 *
 * @return TRUE if the initialization succeeds, else FALSE
 **/
gboolean init_plugin (Plugin *p)
{
#define ANAME "pack"
  aname = _("pack");

  p->signature = ANAME VERSION;
  p->menu_name = _("_Pack..");
  p->description = _("Build an archive containing the selected item(s)");
  p->icon = "plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate

  if (p->plugin_cb == NULL)
  {
	  p->plugin_cb = _e2p_pack;  //set plugins menu item function
//	  E2_Action* (*action_register) () = (*p->api_lookup) (E2API_ACTION_REGISTER);
	  //no need to free this
	  gchar *action_name = g_strconcat (PLUGIN,".",aname,NULL);
	  //NULL = no data, FALSE = no argument for the function
//	  (*action_register)
	  e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2p_pack, NULL, FALSE);
	  return TRUE;
  }
  return FALSE;
}
/**
 * @brief unload this plugin
 *
 * @param p pointer to data struct for the plugin
 *
 * @return
 **/
void unload_plugin (Plugin *p)
{
//  if (p->api_lookup != NULL)
//  {
//	  gboolean (*action_unregister) () = (*p->api_lookup) (E2API_ACTION_UNREGISTER);
	  gchar *action_name = g_strconcat (PLUGIN,".",aname,NULL);
//	  (*action_unregister)
	  e2_action_unregister (action_name);
	  g_free (action_name);
//  }
}
